@@ -39,6 +39,8 @@ class Llama:
         n_threads: Optional[int] = None,
         n_batch: int = 8,
         last_n_tokens_size: int = 64,
+        lora_base: Optional[str] = None,
+        lora_path: Optional[str] = None,
         verbose: bool = True,
     ):
         """Load a llama.cpp model from `model_path`.
@@ -57,6 +59,8 @@ class Llama:
             n_threads: Number of threads to use. If None, the number of threads is automatically determined.
             n_batch: Maximum number of prompt tokens to batch together when calling llama_eval.
             last_n_tokens_size: Maximum number of tokens to keep in the last_n_tokens deque.
+            lora_base: Optional path to base model, useful if using a quantized base model and you want to apply LoRA to an f16 model.
+            lora_path: Path to a LoRA file to apply to the model.
             verbose: Print verbose output to stderr.
 
         Raises:
@@ -108,6 +112,21 @@ class Llama:
             self.model_path.encode("utf-8"), self.params
         )
 
+        self.lora_base = lora_base
+        self.lora_path = lora_path
+        if self.lora_path:
+            if llama_cpp.llama_apply_lora_from_file(
+                self.ctx,
+                llama_cpp.c_char_p(self.lora_path.encode("utf-8")),
+                llama_cpp.c_char_p(self.lora_base.encode("utf-8"))
+                if self.lora_base is not None
+                else llama_cpp.c_char_p(0),
+                llama_cpp.c_int(self.n_threads),
+            ):
+                raise RuntimeError(
+                    f"Failed to apply LoRA from lora path: {self.lora_path} to base path: {self.lora_base}"
+                )
+
         if self.verbose:
             print(llama_cpp.llama_print_system_info().decode("utf-8"), file=sys.stderr)
 
@@ -802,6 +821,8 @@ class Llama:
             last_n_tokens_size=self.last_n_tokens_size,
             n_batch=self.n_batch,
             n_threads=self.n_threads,
+            lora_base=self.lora_base,
+            lora_path=self.lora_path,
         )
 
     def __setstate__(self, state):
@@ -819,6 +840,8 @@ class Llama:
             n_threads=state["n_threads"],
             n_batch=state["n_batch"],
             last_n_tokens_size=state["last_n_tokens_size"],
+            lora_base=state["lora_base"],
+            lora_path=state["lora_path"],
             verbose=state["verbose"],
         )
 
